from pact_test.either import *
from pact_test.constants import FAILED


def match(interaction, pact_response):
    return _match_status(interaction, pact_response)\
        .concat(_match_headers, pact_response)\
        .concat(_match_body, pact_response)


def _match_status(interaction, pact_response):
    expected = interaction['response'].get('status')
    actual = pact_response.status

    if expected is None or actual == expected:
        return Right(interaction)
    return Left(_build_error_message('status', expected, actual))


def _match_headers(interaction, pact_response):
    expected = interaction['response'].get('headers', {})
    actual = _to_dict(pact_response.headers)

    insensitive_expected = {k.upper(): v for (k, v) in expected.items()}
    insensitive_actual = {k.upper(): v for (k, v) in actual.items()}

    if _is_subset(insensitive_expected, insensitive_actual):
        return Right(interaction)
    return Left(_build_error_message('headers', expected, actual))


def _match_body(interaction, pact_response):
    expected = interaction['response'].get('body')
    actual = pact_response.body

    if _is_subset(expected, actual):
        return Right(interaction)
    return Left(_build_error_message('body', expected, actual))


def _to_dict(headers):
    d = {}
    for h in headers:
        d[h[0]] = h[1]
    return d


def _build_error_message(section, expected, actual):
    return {
        'actual': actual,
        'status': FAILED,
        'expected': expected,
        'message': section.capitalize() + ' is incorrect'
    }


def _is_subset(expected, actual):
    actual_items = actual.items() if actual else {}
    expected_items = expected.items() if expected else {}

    stripped_actual_items = map(_strip_whitespaces_after_commas, actual_items)
    stripped_expected_items = map(_strip_whitespaces_after_commas, expected_items)

    return all(item in stripped_actual_items for item in stripped_expected_items)


def _strip_whitespaces_after_commas(t):
    k = t[0]
    v = t[1].replace(', ', ',') if type(t[1]) is str else t[1]

    return k, v



REQUEST = 'request'
TEXT = 'text'
JSON = 'application/json'
PASSED = 'PASSED'
FAILED = 'FAILED'
PROVIDER_STATE = 'providerState'
TEST_PARENT = 'ServiceConsumerTest'
MISSING_STATE = 'Missing implementation for state '
MISSING_PACT_HELPER = 'Missing "pact_helper.py" at "'
MISSING_PACT_URI = 'Missing setup for "pact_uri" at '
MISSING_TESTS = 'There are no consumer tests to verify.'
MISSING_SETUP = 'Missing "setup" method in "pact_helper.py".'
MISSING_HAS_PACT_WITH = 'Missing setup for "has_pact_with" at '
MISSING_TEAR_DOWN = 'Missing "tear_down" method in "pact_helper.py".'
EXTEND_PACT_HELPER = 'Pact Helper class must extend pact_test.PactHelper'



import os
import json


class Config(object):
    pact_broker_uri = None
    consumer_tests_path = 'tests/service_consumers'
    provider_tests_path = 'tests/service_providers'
    CONFIGURATION_FILE = '.pact.json'

    def __init__(self):
        user_config_file = self.path_to_user_config_file()
        if os.path.isfile(user_config_file):
            with open(user_config_file) as f:
                self.load_user_settings(f)

    def load_user_settings(self, user_settings_file):
        settings = json.loads(user_settings_file.read())
        [self.custom_or_default(settings, key) for key in settings.keys()]

    def path_to_user_config_file(self):
        return os.path.join(os.getcwd(), self.CONFIGURATION_FILE)

    def custom_or_default(self, user_settings, key):
        setattr(self, key, user_settings.get(key, getattr(self, key)))



from pact_test.either import *
from pact_test.utils.logger import *
from pact_test.config.config_builder import Config
from pact_test.runners.service_consumers.test_suite import ServiceConsumerTestSuiteRunner
from pact_test.runners.service_providers.provider_tests_runner import ProviderTestsRunner


def verify(verify_consumers=False, verify_providers=False):
    config = Config()

    if verify_consumers:
        run_consumer_tests(config)
    if verify_providers:
        run_provider_tests(config)


def run_consumer_tests(config):
    test_results = ServiceConsumerTestSuiteRunner(config).verify()
    for test_result in test_results:
        info(test_result.value) if test_result is Right else error(test_result.value)


def run_provider_tests(config):
    ProviderTestsRunner(config).verify()
